
/**
<p>Titre : RSA </p>
<p>Description : Encodage de données selon le protocole RSA </p>
<p>Copyright : Copyright (c) 2004</p>
@author François Bradette
@version 1.1
version originale de Robert Sedgewick and Kevin Wayne.Copyright © 2004
pris sur le site http://www.cs.princeton.edu/introcs/104crypto/RSA.java.html
*/

import java.io.*;
import java.math.*;
import java.security.*;

public class RSA implements Serializable {
    	private final static BigInteger one = BigInteger.ONE;
    	private final static SecureRandom random = new SecureRandom();
    
    	private BigInteger privateKey = null;
    	private BigInteger publicKey = null;
    	private BigInteger modulus = null;
    
    	private BigInteger p1 = null;
    	private BigInteger p2 = null;
    
    	//Ces variables doivent être initialisé pour l'encrytage de données.
    	private BigInteger modulusE;
    	private BigInteger publicKeyE;
    
	// Taille de la clef
    	private int N;
    	private BigInteger phi0;

    	public RSA(int N) {
		this.N = N;
		// generate an N-bit (roughly) public and private key
		// clés privé
		// p1
		p1 = BigInteger.probablePrime(N / 2, random);
    
		//p2
		p2 = BigInteger.probablePrime(N / 2, random);

		//0
    		phi0 = (p1.subtract(one)).multiply(p2.subtract(one));
    
		//n
		modulus = p1.multiply(p2);
    
		//d
		setPrivateKey();
    
		//e
		publicKey = privateKey.modInverse(phi0);
    
		modulusE = modulus;
		publicKeyE = publicKey;
	}
    
    	//renvois la variable public modulo utilisé pour l'encryption
    	// des donné
    	public BigInteger getModulus() {
    		return modulus;
    	}
    
    	//renvois la variable public publicKey utilisé par d'autre pour l'encryption
    	public BigInteger getPublicKey() {
    		return publicKey;
    	}
    
    	//renvois la variable public publicKey utilisé par d'autre pour l'encryption
    	public BigInteger getPrivateKey() {
    		return privateKey;
    	}

    	public void printAll() {
    		System.out.println("p1 : "+p1.toString());
    		System.out.println("p2 : "+p2.toString());
    		System.out.println("module : "+modulus.toString());
    		System.out.println("phi : "+phi0.toString());
    		System.out.println("publique : "+publicKey.toString());
    		System.out.println("privee : "+privateKey.toString());
    	}

   
	/**
	  @param xor BigInteger
	  Cette méthode est employer pour encrypté et pour décrypté les clées
	  C'est l'avantage du ou exclusif il sufi de répété la même oprération pour
	  encodé et décodé
	**/
	public void xOrClePrive(BigInteger xor)
	{
		xor = xor.pow(4);
		privateKey = privateKey.xor(xor);
	}
    
    
	/**s'assure que privateKey
	  1. n'a aucun autre diviseur que 1
	  2. qu'il est plus grand que le plus grand entre p1 et p2
	  3. qu'il est plus petit que p1*p2
  	*/
    	private void setPrivateKey()
    	{
    		do {
    			privateKey = BigInteger.probablePrime(N / 2, random);
    		}
    		while (privateKey.gcd(phi0).intValue() != 1 ||
    			privateKey.compareTo(modulus) != -1 ||
    			privateKey.compareTo(p1.max(p2)) == -1);
	}
    
    
	/*
	   Encrypte le message avec les clés public
	   pour l'encryption les clés public doivent etre initialisées
	   le message doit être divisé en paket de N / 8 octects ou bytes
    	*/
	public BigInteger encrypt(BigInteger message, BigInteger publicKeyP, BigInteger modulusP)
	{
		if (message == null) throw new IllegalArgumentException("le message est vide !");
		if (message.compareTo(modulusP) !=-1 ) throw new IllegalArgumentException("le message doit etre inferieur au module");
		if ( message.toByteArray().length >= Integer.MAX_VALUE )
			throw new IllegalArgumentException("je ne prends pas en charge de si gros messages");

		if (publicKeyP == null || modulusP == null )
			throw new IllegalArgumentException("La clef et le module doivent etre definis !");

		//	C'est quoi cette condition ????
		// String str_message = new String(message.toByteArray());
		//if (str_message.length() <= (N / 8))
			
		BigInteger rep = message.modPow(publicKeyP, modulusP);
		return rep;
	}
    
	public BigInteger encrypt(String message, BigInteger publicKeyP, BigInteger modulusP) {
		// Convertit la chaine en un nombre 
		BigInteger b =  new BigInteger(message.getBytes());
		System.out.println("Valeur en Gros entiers : "+b);

		return 	encrypt(b, publicKeyP, modulusP);

	}
	
	// Celle ci renvoie la chaine dechiffree	
	public String decrypt(BigInteger encrypted, BigInteger privateKeyP, BigInteger modulusP) {
		BigInteger resu = decypher(encrypted, privateKeyP, modulusP);
		String s = new String (resu.toByteArray());
		return s;
	}

	//Decrypte le message avec les cle privee
	// Ceci est plutot la methode privee de la classe, de l'exterieur,
	// on veut surtout recuperer des octets dechiffres ou des chaines de char
	// mais pas des BigInteger !
	public BigInteger decypher(BigInteger encrypted, BigInteger privateKeyP, BigInteger modulusP)
	{
		if (encrypted == null) throw new IllegalArgumentException("le message est vide !");
		if (encrypted.compareTo(modulusP) !=-1 ) throw new IllegalArgumentException("le message doit etre inferieur au module");
		if ( encrypted.toByteArray().length >= Integer.MAX_VALUE )
			throw new IllegalArgumentException("je ne prends pas en charge de si gros messages");

		if (privateKeyP == null || modulusP == null )
			throw new IllegalArgumentException("La clef et le module doivent etre definis !");
		return encrypted.modPow(privateKeyP, modulusP);
	}
    
    
	public String toString()
	{
		String s = "";
		s += "public = " + publicKey + "\n";
		s += "modulus = " + modulus;
		return s;
	}
   
}

